#!/usr/bin/python

#######################################
### check_cpu_nix version 0.3.5
### created for nagios by nemir!
### nemir@mindless.com

import string, os, sys, re, signal

try:
	from optik import OptionParser
except ImportError:
	from optparse import OptionParser



#######################################
###  Define a set of strings to handle 
###  any info output requirements.


check_cpu_version = "check_cpu (nagios-plugins 1.4.1!!) 0.3.5\n"

intro = "The nagios plugins come with ABSOLUTELY NO WARRANTY.  You may redistribute\ncopies of the plugins under the terms of the GNU General Public License.\nFor more information about these matters, see the file named COPYING.\nCopyright (c) 2004 Ethan Galstad/Karl DeBisschop\n\n"

preamble = "This plugin will check the percent of idle CPU usage on the system it is\nexecuted on and generate an alert if the percentage is below\none of the threshold values.\n\n"

use = "Usage:\tcheck_cpu -w limit -c limit [-t timeout]\n\tcheck_cpu (-h|--help)\n\tcheck_cpu (-V|--version)\n"

options = "\n -w, --warning=PERCENT\n\tExit with WARNING status if idle CPU percentage is less than PERCENT\n -c, --critical=PERCENT\n\tExit with CRITICAL status if idle CPU percentage is less than PERCENT\n -t, --timeout=INTEGER\n\tSeconds before check attempt times out (default: 10)\n -s, --sample=INTEGER\n\tSeconds to use as sample time. (default: 5)\n -h, --help\n\tPrint detailed help screen\n -V, --version\n\tPrint version information\n\n"

bugs = "Bugs:  Please check the FAQs before reporting errors.  Python related issues\nare all covered by several FAQs.  This does not support non-unix based systems` \nsyntax. One day soon I will figure out how to allow a % sign in the parameters.\nThis has been tested on RedHat 8.0+, Fedora Core 1 & 2, OpenBSD 3.x and Debian.\nI would appreciate feedback and/or patches if it doesn't function under your \nchosen operating system.\n\n"

query = "Send email to nagios-users@lists.sourceforge.net if you have questions\nregarding the use of this software.  To submit patches or suggest improvements,\nsend email to nagiosplug-devel@lists.sourceforge.net.\n\nFor questions and suggestions pertaining to the check_cpu plugin,\nplease send email to nemir@mindless.com.\n" 

fullHelp = check_cpu_version + intro + preamble + use +  options + bugs + query



#######################################
###  Define functions to be called for
###  help or version requests.


def helpFunc(option, opt, value, parser):
 	print fullHelp
	sys.exit(3)

def verFunc(option, opt, value, parser):
	print check_cpu_version
	sys.exit(3)



#######################################
###  Parse all the parameters.  Define
###  variables for later use.


parser = OptionParser(add_help_option=0)

parser.add_option("-h", "--help", action="callback", callback=helpFunc)
parser.add_option("-V", "--version", action="callback", callback=verFunc)
parser.add_option("-w", "--warning", action="store", type="int", dest="warn", default=-1)
parser.add_option("-c", "--critical", action="store", type="int", dest="crit", default=-2)
parser.add_option("-s", "--sample", action="store", type="int", dest="sample", default=5)
parser.add_option("-t", "--timeout", action="store", type="int", dest="timeout", default=10)


critical = options.crit
warning = options.warn
sample = options.sample
timeout = options.timeout


(options, args) = parser.parse_args()
if args: 
	strArgs = string.join(args)
	print strArgs + ": Incorrect parameters specified or keyboard error.\n" + use
	sys.exit(3)

if -3 > critical or 101 < critical:
	print "Critical value is a percentage and must be between 0 and 100.\n" + use
	sys.exit(3)

if -2 > warning or 101 < warning:
	print "Warning value is a percentage and must be between 0 and 100.\n" + use
	sys.exit(3)

if critical >= warning:
	print "Critical value must be less than the warning value.\n" + use 
	sys.exit(3)

if sample >= timeout:
	print "Sample time must be less than timeout.\n" + use 
	sys.exit(3)


strSample = str(sample)



#######################################
###  Define the signal handler using
###  the timeout value.

def handler(signum, frame):
	print "Timeout value exceeded. Could not determine CPU usage."
	sys.exit(3)

signal.signal(signal.SIGALRM, handler)
signal.alarm(timeout)



#######################################
### Determine what OS we are being run
### on, to figure syntax required.


v = os.popen("uname", "r")
l = v.read()

if l == "Linux\n":
	vmcmd = "/usr/bin/vmstat " + strSample + " 2"
elif l == "OpenBSD\n":
	vmcmd = "/usr/bin/vmstat -c 2 -w " + strSample 
elif l == "FreeBSD\n":
	vmcmd = "/usr/bin/vmstat -c 2 -w " + strSample 
elif l == "NetBSD\n":
	vmcmd = "/usr/bin/vmstat -c 2 -w " + strSample 
else:
	print "Unknown operating system, unable to continue.\n"
	sys.exit(3)



#######################################
### Grab the CPU sample and convert the 
### the relevent info to an int.


vmstat = os.popen( vmcmd, "r" )
statOut = vmstat.read()

statLines = string.split( statOut, '\n')
lastLine = statLines[-2]

valueStr = re.sub( "[ \t\n\r\v]+", " ", lastLine )
values = string.split( valueStr, ' ')


idleCPU = values[-1]
sysCPU = values[-2]
userCPU = values[-3]

idleValue = string.atoi(idleCPU)



#######################################
### Finally, determine and report CPU
### state,  and exit the plugin.


if idleValue <= critical:
	status = "CPU CRITICAL"
	print status + ": CPU is only " + idleCPU + "% idle"
	sys.exit(2) 
elif idleValue <= warning:
	status = "WARNING"
	print status + ": CPU is only " + idleCPU + "% idle"
	sys.exit(1)
elif warning < idleValue:
	status = "OK"
	print status + ": CPU is " + idleCPU + "% idle"
	sys.exit(0)
else:
	status = "CPU STATUS UNKNOWN"
	print status + ": Could not complete check."
	sys.exit(3)


signal.alarm(0)

